#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-11-18 13:54:56 +0000 (Tue, 18 Nov 2008) $
# Id:            $Id: bustard_output.pm 3510 2008-11-18 13:54:56Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-5.0/lib/npg_qc/api/process/bustard_output.pm $
#

package npg_qc::api::process::bustard_output;

use strict;
use warnings;
use Class::Std;
use Readonly;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::process);
use Statistics::Lite qw(:all);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3510 $ =~ /(\d+)/mx; $r; };

Readonly our $THREE                       => 3;
Readonly our $FOUR                        => 4;
Readonly our $FIVE                        => 5;
Readonly our $QVAL_TWENTY                 => 20;
Readonly our $QVAL_TWENTYFIVE             => 25;
Readonly our $QVAL_THIRTY                 => 30;
Readonly our $POSITION_OF_QVAL_TWENTY     => 26;
Readonly our $POSITION_OF_QVAL_TWENTYFIVE => 31;
Readonly our $POSITION_OF_QVAL_THIRTY     => 36;
Readonly our $PERCENTAGE_CALCULATOR       => 1000;

{
  ## no critic
  my %file_of     :ATTR( :get<file>,     :set<file> );
  my %rescore_of  :ATTR( :get<rescore>,  :set<rescore> );
  my %position_of :ATTR( :get<position>, :set<position> );
  my %tile_of     :ATTR( :get<tile>,     :set<tile> );
  my %end_of      :ATTR( :get<end>,      :set<end> );
  ## use critic

  sub convert_to_xml {
    my ($self, $file_url) = @_;
    my $r;

    eval{
      $r = $file_url =~ /\.gz\z/xms              ? croak qq{$file_url is not the correct file_type to process}
         : $file_url =~ /score\.txt/xms          ? $self->tile_score_xml($file_url)
         : $file_url =~ /_all\.txt/xms           ? $self->tile_all_xml($file_url)
         : $file_url =~ /config\.txt/xms         ? $self->config_xml($file_url)
         : $file_url =~ /\/Signal_Means\.txt/xms ? $self->signal_means_xml($file_url)
         : $file_url =~ /Summary\.htm/xms        ? $self->summary_data_xml($file_url)
         : $file_url =~ /qcalreport\.txt/xms     ? $self->qcalreport_xml($file_url)
         : $file_url =~ /fastqcheck/xms          ? $self->fastqcal_xml($file_url)
         :                                         croak qq{$file_url is not the correct file_type to process}
         ;
      if ($file_url !~ /config\.txt/xms) {
        $self->util->parser->parse_string($r); # sanity checks the xml is parsable
      }
      1;
    } or do {
      return;
    };
    return $r;
  }

  sub process_file {
    my ($self, $file_url) = @_;
    my $capture = $self->process_id_run($file_url);

    if (($capture) = $file_url =~ /s_(\d{1})_.*?\.txt/xms) {
      $self->set_position($capture);
    }

    if ($file_url =~ /fastqcheck/xms) {
      ($capture) = $file_url =~ /s_(\d{1})\.fastqcheck/xms;
      $self->set_position($capture);
    }

    if (($capture) = $file_url =~ /s_\d{1}_0*(\d+)_.*?\.txt/xms) {
      $self->set_tile($capture);
    }

    if (($capture) = $file_url =~ /s_\d{1}_(\d{1})_qcalreport.txt/xms) {
      $capture ||= 1;
      $self->set_end($capture);
    }
  
    my $rescore = $file_url =~ /s_\d{1}_\d+_rescore.txt/xms ? $self->set_rescore(1)
                :                                             $self->set_rescore(0)
                ;
    my $file = $self->read_file($file_url);
    $self->set_file($file);

    return;
  }

  sub rescore {
    my ($self, $file_url) = @_;

    if (! defined $self->get_rescore()) {
      $self->process_file($file_url);
    }

    return $self->get_rescore();
  }

  sub file {
    my ($self, $file_url) = @_;

    if (!$self->get_file()) {
      $self->process_file($file_url);
    }

    return $self->get_file();
  }

  sub position {
    my ($self, $file_url) = @_;

    if (!$self->get_position()) {
      $self->process_file($file_url);
    }

    return $self->get_position();
  }

  sub id_run {
    my ($self, $file_url) = @_;

    if (!$self->get_id_run()) {
      $self->process_file($file_url);
    }

    return $self->get_id_run();
  }

  sub tile {
    my ($self, $file_url) = @_;

    if (!$self->get_tile()) {
      $self->process_file($file_url);
    }

    return $self->get_tile();
  }

  sub end {
    my ($self, $file_url) = @_;

    if (!$self->get_end()) {
      $self->process_file($file_url);
    }

    return $self->get_end();
  }

  sub config_xml {
    my ($self, $file_url) = @_;

    my $file   = $self->file($file_url);
    my $id_run = $self->id_run($file_url);

    return qq{<?xml version="1.0" encoding="utf-8"?><root>\n<dataset>run_config</dataset><run_config id_run="$id_run" config_text="$file" /></root>};
  }

  sub qcalreport_xml {
    my ($self, $file_url) = @_;

    my $file     = $self->file($file_url);
    my $id_run   = $self->id_run($file_url);
    my $position = $self->position($file_url);
    my $end      = $self->end($file_url);

    my $xml = qq{<?xml version="1.0" encoding="utf-8"?><root>\n<dataset>qcal</dataset><row id_run="$id_run" position="$position" end="$end" };

    $file =~ s/.*Results\ for\ quality\ filtered\ data\n//xms;

    my @lines = split /\n/xm, $file;
    splice @lines, 0, $FOUR;
    my $wanted = {
      $QVAL_TWENTY     => 'twenty',
      $QVAL_TWENTYFIVE => 'twentyfive',
      $QVAL_THIRTY     => 'thirty',
    };
    foreach my $line (@lines) {
      my @temp = split /\s+/xm, $line;
      if ( $wanted->{$temp[0]} ) {
        $xml .= qq{$wanted->{$temp[0]}="$temp[2]" };
      }
    }

    $xml .= q{ /></root>};

    return $xml;
  }

  sub fastqcal_xml {
    my ($self, $file_url) = @_;

    my $file     = $self->file($file_url);
    my $id_run   = $self->id_run($file_url);
    my $position = $self->position($file_url);
    my $xml = qq{<?xml version="1.0" encoding="utf-8"?><root>\n<dataset>qcal</dataset><row id_run="$id_run" position="$position" end="1" };

    my @lines = split /\n/xm, $file;
    my @temp = split /\s+/xm, $lines[0];
    my $pf_bases = $temp[2];

    foreach my $line (@lines) {
      @temp = split /\s+/xm, $line;
      if ($temp[0] eq 'Total') {
        pop @temp;
        my @working_array = @temp;
        splice @working_array, 0, $POSITION_OF_QVAL_TWENTY;
        my $perc_at_20plus = sum(@working_array) || 0;
        my $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_20plus/$PERCENTAGE_CALCULATOR;
        $calculated_value ||= 0;
        $xml .= qq{twenty="$calculated_value" };
        @working_array = @temp;
        splice @working_array, 0, $POSITION_OF_QVAL_TWENTYFIVE;
        my $perc_at_25plus = sum(@working_array) || 0;
        $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_25plus/$PERCENTAGE_CALCULATOR;
        $calculated_value ||= 0;
        $xml .= qq{twentyfive="$calculated_value" };
        @working_array = @temp;
        splice @working_array, 0, $POSITION_OF_QVAL_THIRTY;
        my $perc_at_30plus = sum(@working_array) || 0;
        $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_30plus/$PERCENTAGE_CALCULATOR;
        $calculated_value ||= 0;
        $xml .= qq{thirty="$calculated_value" };
        last;
      }
    }
    
    $xml .= q{/></root>};
    return $xml;
  }

  sub tile_all_xml {
    my ($self, $file_url) = @_;

    my @lines    = split /\n/xms, $self->file($file_url);
    my $id_run   = $self->id_run($file_url);
    my $tile     = $self->tile($file_url);
    my $position = $self->position($file_url);
    my $end = 1;

    my $xml      = qq{<?xml version="1.0" encoding="utf-8"?>\n<root>\n<dataset>tile_all</dataset>\n<run_tile id_run="$id_run" tile="$tile" position="$position" end="$end" />\n<tile_all>\n};

    foreach my $line (@lines) {
      next if ($line =~ /\A\#/xms);
      my @temp = split q{ }, $line;
      shift @temp;
      $xml .= qq{<row cycle="$temp[0]" all_a="$temp[1]" all_c="$temp[2]" all_g="$temp[3]" all_t="$temp[4]" call_a="$temp[5]" call_c="$temp[6]" call_g="$temp[7]" call_t="$temp[8]" num_a="$temp[9]" num_c="$temp[10]" num_g="$temp[11]" num_t="$temp[12]" num_x="$temp[13]" />\n}; ## no critic
    }

    $xml .= qq{</tile_all>\n</root>};

    return $xml;
  }

  sub signal_means_xml {
    my ($self, $file_url) = @_;

    my @lines  = split /\n/xms, $self->file($file_url);
    my $id_run = $self->id_run($file_url);

    my $xml    = qq{<?xml version="1.0" encoding="utf-8"?>\n<root>\n<dataset>signal_mean</dataset>\n<run id_run="$id_run">\n  <lanes>\n};

    my $position;
    foreach my $line (@lines) {
      next if ($line =~ /\A\#/xms);
      my @temp = split q{ }, $line;
      my $lane = shift @temp;
      my $append_xml = !$position         ? qq{    <lane position="$lane">\n      <signal_means>\n}
                     : $lane ne $position ? qq{      </signal_means>\n    </lane>\n    <lane position="$lane">\n      <signal_means>\n}
                     :                      q{}
                     ;
      $xml .= $append_xml;
      $xml .= qq{        <row cycle="$temp[0]" all_a="$temp[1]" all_c="$temp[2]" all_g="$temp[3]" all_t="$temp[4]" call_a="$temp[5]" call_c="$temp[6]" call_g="$temp[7]" call_t="$temp[8]" base_a="$temp[9]" base_c="$temp[10]" base_g="$temp[11]" base_t="$temp[12]" />\n}; ## no critic
      $position = $lane;
    }

    $xml .= qq{      </signal_means>\n    </lane>\n  </lanes>\n</run>\n</root>};

    return $xml;
  }

  sub tile_score_xml {
    my ($self, $file_url) = @_;

    my $file     = $self->file($file_url);
    my $id_run   = $self->id_run($file_url);
    my $tile     = $self->tile($file_url);
    my $position = $self->position($file_url);
    my $rescore  = $self->rescore($file_url);

    my $xml      = qq{<?xml version="1.0" encoding="utf-8"?>\n<root>\n<dataset>tile_score</dataset>\n<run_tiles>\n};

    my $end      = 0;
    my @ends;
    if ($file !~ /bases\ of\ sequence\ found.*?bases\ of\ sequence\ found.*?bases\ of\ sequence\ found/xms) {

      push @ends, $file;

    } else {

      @ends = split /RUN_TIME/xms, $file;
      shift @ends;

      foreach my $text (@ends) {
        $text = '#RUN_TIME'.$text;
      }

    }
    
    foreach my $text (@ends) {

      last if ($end == 2);

      $end++;
      $xml .= qq{<run_tile id_run="$id_run" tile="$tile" position="$position" end="$end">\n};
      $xml .= $self->tile_score($text);

      my @lines = split /\n/xms, $text;
      my ($cumulative_errors_by_cycle, $log_likelihood, $information_content_by_cycle, $error_rate_relative_reference_cycle_nucleotide);

      foreach my $line (@lines) {
        my $processed = $line =~ /\A\!\d+/xms ? push @{$cumulative_errors_by_cycle}, $line
                      : $line =~ /\A\>\d+/xms ? push @{$log_likelihood}, $line
                      : $line =~ /\A\~\d+/xms ? push @{$information_content_by_cycle}, $line
                      : $line =~ /\A\@\d+/xms ? push @{$error_rate_relative_reference_cycle_nucleotide}, $line
                      :                         q{}
                      ;
      }

      $xml .= $self->cumulative_errors_by_cycle($cumulative_errors_by_cycle);
      $xml .= $self->log_likelihood($log_likelihood);
      $xml .= $self->information_content_by_cycle($information_content_by_cycle);
      $xml .= $self->error_rate_relative_reference_cycle_nucleotide($error_rate_relative_reference_cycle_nucleotide);
      $xml .= $self->errors_by_cycle($text);
      $xml .= $self->error_rate_relative_to_reference_base_inc_blanks($text);
      $xml .= $self->error_rate_relative_to_reference_base_no_blanks($text);
      $xml .= $self->error_rate_relative_sequence_base($text);
      $xml .= $self->errors_by_nucleotide($text);
      $xml .= $self->errors_by_cycle_and_nucleotide($text);
      $xml .= $self->most_common_word($text);
      $xml .= $self->most_common_blank_pattern($text);

      $xml .= qq{</run_tile>\n};

    }

    $xml .= qq{</run_tiles>\n</root>};

    return $xml;
  }

  sub tile_score {
    my ($self, $file) = @_;

    $file =~ s/Breakdown\ of\ errors\ by\ cycle.*//gxms;

    my $rescore  = $self->rescore();

    my ($base_count)              = $file =~ /(\d+)\ bases\ of\ sequence\ found/xms;
    my ($error_count)             = $file =~ /(\d+)\ were\ errors/xms;
    my ($blank_count)             = $file =~ /(\d+)\ were\ blanks/xms;
    my ($unique_alignments)       = $file =~ /unique\ alignments\ :\ (\d+)/xms;
    my ($ua_total_score)          = $file =~ /total\ score\ (\d+)/xms;
    my ($cycles)                  = $file =~ /cycles\ :\ (\d+)/xms;
    my ($score_version)           = $file =~ /score\.pl\,v\ (\d+\.\d+)/xms;
    my ($phagealign_version)      = $file =~ /PhageAlign\.cpp\,v\ (\d+\.\d+)/xms;
    my ($max_blanks)              = $file =~ /\#MAX_BLANKS\ (\d+)/xms;
    my ($seq_length)              = $file =~ /\#SEQ_LENGTH\ (\d+)/xms;
    my ($genome_file)             = $file =~ /\#GENOME_FILE\ (.*?)\n/xms;
    my ($bases_used)              = $file =~ /\#BASES_USED\ (.*?)\n/xms;
    my ($qualityfilter_version)   = $file =~ /qualityFilter\.pl\,v\ (\d+\.\d+)/xms;
    my ($filter_criterion)        = $file =~ /\#FILTER_CRITERION\ \(\((.*?)\)\)/xms;
    my $date_run = {
      score         => q{},
      phagealign    => q{},
      qualityfilter => q{},
    };

    $file =~ s/\#MAX_BLANKS.*//gxms;
    my @lines = split /\n/xms, $file;

    for my $i (0..scalar@lines) {

      next if (!$lines[$i]);

      my $program = $lines[$i] =~ /PhageAlign/xms    ? q{phagealign}
                  : $lines[$i] =~ /score\.pl/xms     ? q{score}
                  : $lines[$i] =~ /qualityFilter/xms ? q{qualityfilter}
                  :                                    next
                  ;

      my $date_row = $i - 1;
      my @temp = split q{ }, $lines[$date_row];
      my $months = { Jan => '01', Feb => '02', Mar => '03', Apr => '04', May => '05', Jun => '06', Jul => '07', Aug => '08', Sep => '09', Oct => '10', Nov => '11', Dec => '12' };
      $temp[3] = sprintf '%02d', $temp[3]; ## no critic
      $date_run->{$program} = $temp[-1] . q{-} . $months->{$temp[2]} . q{-} . $temp[3] . q{ } . $temp[4]; ## no critic

    }
    
    $genome_file            ||= q{};
    $seq_length             ||= q{};
    $max_blanks             ||= q{};
    $phagealign_version     ||= q{};
    $bases_used             ||= q{};
    $qualityfilter_version  ||= q{};
    $filter_criterion       ||= q{};

    return qq{<tile_score base_count="$base_count" error_count="$error_count" blank_count="$blank_count" unique_alignments="$unique_alignments" ua_total_score="$ua_total_score" cycles="$cycles" rescore="$rescore" score_version="$score_version" score_date_run="$date_run->{score}" phagealign_version="$phagealign_version" phagealign_date_run="$date_run->{phagealign}" max_blanks="$max_blanks" seq_length="$seq_length" genome_file="$genome_file" bases_used="$bases_used" qualityfilter_version="$qualityfilter_version" qualityfilter_date_run="$date_run->{qualityfilter}" filter_criterion="$filter_criterion" />\n};
  }

  sub cumulative_errors_by_cycle {
    my ($self, $table_rows) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<cumulative_errors_by_cycle>\n};

    foreach my $row (@{$table_rows}) {

      my @temp = split q{ }, $row;
      ($temp[0]) = $temp[0] =~ /(\d+)/xms;
      $xml .= qq{  <row cycle="$temp[0]" one="$temp[1]" two="$temp[2]" three="$temp[3]" four="$temp[4]" five="$temp[5]" rescore="$rescore" />\n} ## no critic

    }

    $xml .= qq{</cumulative_errors_by_cycle>\n};

    return $xml;
  }

  sub log_likelihood {
    my ($self, $table_rows) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<log_likelihood>\n};

    foreach my $row (@{$table_rows}) {

      my @temp = split q{ }, $row;
      ($temp[0]) = $temp[0] =~ /(\d+)/xms;
      $xml .= qq{  <row cycle="$temp[0]" read_as="$temp[1]" really_a="$temp[2]" really_c="$temp[3]" really_g="$temp[4]" really_t="$temp[5]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</log_likelihood>\n};

    return $xml;
  }

  sub information_content_by_cycle {
    my ($self, $table_rows) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<information_content_by_cycle>\n};

    foreach my $row (@{$table_rows}) {

      my @temp = split q{ }, $row;
      ($temp[0]) = $temp[0] =~ /(\d+)/xms;
      $xml .= qq{  <row cycle="$temp[0]" equiv_info="$temp[1]" align="$temp[2]" total="$temp[3]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</information_content_by_cycle>\n};

    return $xml;
  }

  sub error_rate_relative_reference_cycle_nucleotide {
    my ($self, $table_rows) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<error_rate_relative_reference_cycle_nucleotide>\n};

    foreach my $row (@{$table_rows}) {

      my @temp = split q{ }, $row;
      ($temp[0]) = $temp[0] =~ /(\d+)/xms;
      $xml .= qq{  <row cycle="$temp[0]" read_as="$temp[1]" really_a="$temp[2]" really_c="$temp[3]" really_g="$temp[4]" really_t="$temp[5]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</error_rate_relative_reference_cycle_nucleotide>\n};

    return $xml;
  }

  sub errors_by_cycle {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<errors_by_cycle>\n};

    my ($wanted) = $file =~ /Breakdown\ of\ errors\ by\ cycle\n(.*?)\n\nError\ rate\ relative\ to\ reference\ base/xms;
    my @lines = split /\n/xms, $wanted;
    shift @lines;

    foreach my $line (@lines) {

      my @temp =  split /\s+/xms, $line;
      $xml .= qq{  <row cycle="$temp[0]" error_percentage="$temp[1]" blank_percentage="$temp[2]" rescore="$rescore" />\n};

    }

    $xml .= qq{</errors_by_cycle>\n};

    return $xml;
  }

  sub error_rate_relative_to_reference_base_inc_blanks {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<error_rate_reference_including_blanks>\n};

    my ($wanted) = $file =~ /Error\ rate\ relative\ to\ reference\ base\ \(including\ blanks\)\n(.*?)\nError\ rate\ relative\ to\ reference\ base\ \(excluding\ blanks\)/xms;
    my @lines = split /\n/xms, $wanted;
    splice @lines, 0, $FOUR;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      $xml .= qq{  <row really="$temp[0]" read_as_blank="$temp[1]" read_as_a="$temp[2]" read_as_c="$temp[3]" read_as_g="$temp[4]" read_as_t="$temp[5]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</error_rate_reference_including_blanks>\n};

    return $xml;
  }

  sub error_rate_relative_to_reference_base_no_blanks {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<error_rate_reference_no_blanks>\n};

    my ($wanted) = $file =~ /Error\ rate\ relative\ to\ reference\ base\ \(excluding\ blanks\)\n(.*?)\nError\ rate\ relative\ to\ sequenced\ base/xms;
    my @lines = split /\n/xms, $wanted;
    splice @lines, 0, $FOUR;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      $xml .= qq{  <row really="$temp[0]" read_as_a="$temp[1]" read_as_c="$temp[2]" read_as_g="$temp[3]" read_as_t="$temp[4]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</error_rate_reference_no_blanks>\n};

    return $xml;
  }

  sub error_rate_relative_sequence_base {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<error_rate_relative_sequence_base>\n};

    my ($wanted) = $file =~ /Error\ rate\ relative\ to\ sequenced\ base\n(.*?)\nBreakdown\ of\ errors\ by\ nucleotide/xms;
    my @lines = split /\n/xms, $wanted;
    splice @lines, 0, $FOUR;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      $xml .= qq{  <row read_as="$temp[0]" really_a="$temp[1]" really_c="$temp[2]" really_g="$temp[3]" really_t="$temp[4]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</error_rate_relative_sequence_base>\n};

    return $xml;
  }

  sub errors_by_nucleotide {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<errors_by_nucleotide>\n};

    my ($wanted) = $file =~ /Breakdown\ of\ errors\ by\ nucleotide\n(.*?)\nFull\ breakdown\ of\ errors\ by\ cycle\ and\ nucleotide/xms;
    my @lines = split /\n/xms, $wanted;
    splice @lines, 0, $THREE;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      $xml .= qq{  <row read_as="$temp[0]" really_a="$temp[1]" really_c="$temp[2]" really_g="$temp[3]" really_t="$temp[4]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</errors_by_nucleotide>\n};

    return $xml;
  }

  sub errors_by_cycle_and_nucleotide {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<errors_by_cycle_and_nucleotide>\n};

    my ($wanted) = $file =~ /Full\ breakdown\ of\ errors\ by\ cycle\ and\ nucleotide\n(.*?)\nError\ rate\ relative\ to\ reference\ by\ cycle\/nucleotide/xms;
    my @lines = split /\n/xms, $wanted;
    splice @lines, 0, 2;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      $xml .= qq{  <row cycle="$temp[0]" read_as="$temp[1]" really_a="$temp[2]" really_c="$temp[3]" really_g="$temp[4]" really_t="$temp[5]" rescore="$rescore" />\n}; ## no critic

    }

    $xml .= qq{</errors_by_cycle_and_nucleotide>\n};

    return $xml;
  }

  sub most_common_word {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<most_common_word>\n};

    my ($wanted) = $file =~ /The\ \d+\ most\ common\ words\ with\ 2\ blanks\ or\ less\:\n(.*?)\n\nThe/xms;
    my @lines;
    if ($wanted) {
      @lines = split /\n/xms, $wanted;
    }
    shift@lines;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      my $ranking = shift @temp;
      my $occurrence = shift @temp;

      my $number_of_sequences;
      if ($temp[0] =~ /\A\d+\z/xms) {
        $xml .= qq{  <row rank="$ranking" occurrence="$occurrence" word="" number_of_sequences="$temp[0]" rescore="$rescore" />\n};
        next;
      }

      $number_of_sequences = scalar @temp;

      foreach my $word (@temp) {
        $xml .= qq{  <row rank="$ranking" occurrence="$occurrence" word="$word" number_of_sequences="$number_of_sequences" rescore="$rescore" />\n};
      }

    }

    $xml .= qq{</most_common_word>\n};

    return $xml;
  }

  sub most_common_blank_pattern {
    my ($self, $file) = @_;

    my $rescore = $self->rescore();

    my $xml = qq{<most_common_blank_pattern>\n};

    my ($wanted) = $file =~ /\(N=any\ nonblank\ character\)\n(.*?)\n\nLog/xms;
    my @lines;
    if ($wanted) {
      @lines = split /\n/xms, $wanted;
    }
    shift@lines;

    foreach my $line (@lines) {

      my @temp = split /\s+/xms, $line;
      my $ranking = shift @temp;
      my $occurrence = shift @temp;

      my $number_of_sequences;
      if ($temp[0] =~ /\A\d+\z/xms) {
        $xml .= qq{  <row rank="$ranking" occurrence="$occurrence" pattern="" number_of_sequences="$temp[0]" rescore="$rescore" />\n};
        next;
      }

      $number_of_sequences = scalar @temp;

      foreach my $word (@temp) {
        $xml .= qq{  <row rank="$ranking" occurrence="$occurrence" pattern="$word" number_of_sequences="$number_of_sequences" rescore="$rescore" />\n};
      }

    }

    $xml .= qq{</most_common_blank_pattern>\n};

    return $xml;
  }

  sub reprocess_summary {
    my ($self, $xml) = @_;

    my $parsed_xml   = $self->util->parser->parse_string($xml);
    my $dataset      = $parsed_xml->getElementsByTagName('dataset')->[0]->toString();
    my $run          = $parsed_xml->getElementsByTagName('run')->[0]->toString();
    my $chip_summary = $parsed_xml->getElementsByTagName('chip_summary')->[0]->toString();

    my @lanes = $parsed_xml->getElementsByTagName('lane');
    
    foreach my $lane (@lanes) {
      $lane = $lane->toString();
      $lane = q{<?xml version="1.0" encoding="utf-8"?><root>} . $dataset . $run . $chip_summary . q{<lane_info>} . $lane . q{</lane_info></root>};
      eval { $self->util->parser->parse_string($lane); } or do { croak $EVAL_ERROR; };
    }

    return @lanes;
 }

  sub summary_data_xml {
    my ($self, $file_url) = @_;
    my $parser = $self->util->parser();
    my $file   = $self->file($file_url);
    my $id_run = $self->id_run($file_url);
    my $paired = 0;

    if ($file =~ /Read\ 1/xms) {
      $paired = 1;
    }

    $file =~ s/\<br>/<br\ \/>/gxms; # corrects for poorly formatted html
    $file =~ s/\<\/br\>/<br\ \/>/gxms; # corrects for poorly formatted html
    $file =~ s/\<br\>/<br\ \/>/gxms; # corrects for poorly formatted html
    $file =~ s/\<=/&lt;=/gxms; # corrects for poorly formatted html
    ($file) = $file =~ /(\<body\>.*\<\/body\>)/xms; # just gets main body, as nothing useful in head and ensures that none of the badly formed head html is parsed (or more like failed to be parsed)
    my $xml    = qq{<?xml version="1.0" encoding="utf-8"?>\n<root>\n<dataset>summary_data</dataset>\n<run id_run="$id_run" paired="$paired" />\n};
    my $body;
    eval {
      $body = $parser->parse_string($file);
    } or do {
      croak $EVAL_ERROR;
    };

    my @tables = $body->getElementsByTagName('table');
    my @rows = $tables[0]->getElementsByTagName('tr');
    my @fields = $rows[0]->getElementsByTagName('td');

    my @chip_summary_tables = splice @tables, 0, 2;
    $xml .= $self->chip_summary(@chip_summary_tables, $paired);

    $xml .= q{<lane_info>};

    my $read = 1;

    if ($paired) {
      splice @tables, 0, $FIVE;
      while (@tables) {
        my $table = shift @tables;
        $xml .= $self->lane_info($table, $read);
        $read = $read == 1 ? 2
              :              1
              ;
      }
    } else {
      splice @tables, 0, $THREE;
      while (@tables) {
        my $table = shift @tables;
        $xml .= $self->lane_info($table, $read);
      }
    }

    $xml .= qq{</lane_info>\n</root>};
    return $xml;
  }

  sub lane_info {
    my ($self, $table, $read) = @_;

    my @rows = $table->getElementsByTagName('tr');
    my $row_headers = shift @rows;
    my @row_headers = $row_headers->getElementsByTagName('td');

    return q{} if ( scalar@row_headers < 1 );

    foreach my $header (@row_headers) {
      my $temp = lc$header->textContent();
      $temp =~ s/\ /_/gxms;
      $temp =~ s/_\z//gxms;
      $temp =~ s/\(//gxms;
      $temp =~ s/\)//gxms;
      $temp =~ s/\%/perc/gxms;
      $header = $temp;
    }

    my $xml = q{<lane>};

    foreach my $row (@rows) {

      my @fields = $row->getElementsByTagName('td');

      $xml .= qq{<row end="$read" };

      my $to = scalar@fields - 1;

      for my $i (0..$to) {

        next if (!$row_headers[$i]);

        my $value = 'NULL';
        if ($fields[$i]) {
	  $value = $fields[$i]->textContent();
	};
	$value =~ s/\[//gxms;
	$value =~ s/\]//gxms;

	if ($value eq 'N/A') {
	  $value = 0;
	}

        $xml .= qq{$row_headers[$i]="$value" };

      }

      $xml .= q{/>};

    }

    $xml .= q{</lane>};

    return $xml;
  }

  sub chip_summary {
    my ($self, $table_1, $table_2, $paired) = @_;

    my %chip_summary;

    my @rows = $table_1->getElementsByTagName('tr');

    foreach my $row (@rows) {

      my @fields = $row->getElementsByTagName('td');
      my $fieldname = lc$fields[0]->textContent();
      $fieldname =~ s/\ /_/gxms;
      $chip_summary{$fieldname} = $fields[1]->textContent();

    }

    @rows      = $table_2->getElementsByTagName('tr');
    my @fields = $rows[1]->getElementsByTagName('td');

    $chip_summary{clusters}    = $fields[0]->textContent();
    $chip_summary{clusters_pf} = $fields[1]->textContent();
    $chip_summary{yield_kb}    = $fields[2]->textContent();

    my $xml = q{<chip_summary><row };

    foreach my $fieldname (sort keys %chip_summary) {
      $xml .= qq{$fieldname="$chip_summary{$fieldname}" };
    }

    $xml .= qq{paired="$paired" };

    $xml .= qq{/></chip_summary>\n};

    return $xml;
  }

}

1;
__END__

=head1 NAME

npg_qc::api::process::bustard_output

=head1 VERSION

$LastChangedRevision: 3510 $

=head1 SYNOPSIS

  my $oBustardOutput = npg_qc::api::process::bustard_output->new({util => $oUtil});

=head1 DESCRIPTION

Processes bustard output files into xml format

=head1 SUBROUTINES/METHODS

=head2 convert_to_xml - processes file and returns xml containing the relevant information

  my $sXML = $oBustardOutput->convert_to_xml($full_path_uri_to_file)

=head2 process_file - processes the $full_path_uri_to_file in order to obtain information about rescore, file, position, id_run and tile. stores these

=head2 rescore  - accessor for rescore
=head2 file     - accessor for file contents
=head2 position - accessor for lane position on flowcell
=head2 id_run   - accessor for id_run
=head2 tile     - accessor for tile number
=head2 end      - accessor for run end

=head2 config_xml       - processes the config_text.txt file into xml
=head2 tile_all_xml     - processes the tile_all.txt file into xml
=head2 signal_means_xml - processes the signal_means.txt file into xml
=head2 tile_score_xml   - processes the tile_score.txt or tile_rescore.txt file into xml
=head2 summary_data_xml - processes Summary.htm into xml
=head2 qcalreport_xml   - processes the qcalreport.txt file into xml
=head2 fastqcal_xml     - processes the fastqcal file into xml

=head2 chip_summary - creates xml snippet with the chip summary data in it
=head2 lane_info - creates xml snippet with the qc summary data per tile per lane from Summary.htm
=head2 tile_score - creates xml snippet relevant for tile_score table
=head2 cumulative_errors_by_cycle - creates xml snippet relevant for cumulative_errors_by_cycle table
=head2 log_likelihood - creates xml snippet relevant for log_likelihood table
=head2 information_content_by_cycle - creates xml snippet relevant for information_content_by_cycle table
=head2 error_rate_relative_reference_cycle_nucleotide - creates xml snippet relevant for error_rate_relative_reference_cycle_nucleotide table
=head2 errors_by_cycle - creates xml snippet relevant for errors_by_cycle table
=head2 error_rate_relative_to_reference_base_inc_blanks - creates xml snippet relevant for error_rate_relative_to_reference_base_inc_blanks table
=head2 error_rate_relative_to_reference_base_no_blanks - creates xml snippet relevant for error_rate_relative_to_reference_base_no_blanks table
=head2 error_rate_relative_sequence_base - creates xml snippet relevant for error_rate_relative_sequence_base table
=head2 errors_by_nucleotide - creates xml snippet relevant for errors_by_nucleotide table
=head2 errors_by_cycle_and_nucleotide - creates xml snippet relevant for errors_by_cycle_and_nucleotide table
=head2 most_common_word - creates xml snippet relevant for most_common_word table
=head2 most_common_blank_pattern - creates xml snippet relevant for most_common_blank_pattern table
=head2 reprocess_summary - splits the summary_xml into smaller 1 lane chunks

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::process

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
