#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-19
# Last Modified: $Date: 2008-11-18 13:54:56 +0000 (Tue, 18 Nov 2008) $
# Id:            $Id: run.pm 3510 2008-11-18 13:54:56Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-5.0/lib/npg_qc/api/run.pm $
#
package npg_qc::api::run;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::base);
use Readonly;
use JSON;
use npg_qc::api::run::lane;
use npg_qc::api::summary;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3510 $ =~ /(\d+)/mx; $r; };

{
  ##no critic
  my %id_run_of         :ATTR( :get<id_run>,         :set<id_run>         );
  my %is_paired_of      :ATTR( :get<is_paired>,      :set<is_paired>      );
  my %id_run_pair_of    :ATTR( :get<id_run_pair>,    :set<id_run_pair>    );
  my %read_one_lanes_of :ATTR( :get<read_one_lanes>, :set<read_one_lanes> );
  my %read_two_lanes_of :ATTR( :get<read_two_lanes>, :set<read_two_lanes> );
  my %summary_of        :ATTR( :get<summary>,        :set<summary>        );
  my %all_runs_of       :ATTR( :get<all_runs>,       :set<all_runs>       );
  ##use critic

  sub fields {
    return qw(
              id_run
              is_paired
              id_run_pair
            );
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub read_run_tiles {
    my ($self) = @_;
    my $id_run = $self->id_run();
    my $util = $self->util();
    my $obj_pk     = $self->primary_key();
    my $pk_value   = $self->$obj_pk();
    my $obj_uri    = sprintf '%s/%s', $util->base_uri(), 'run_tile.json?id_run='.$pk_value;
    my $response   = $util->get($obj_uri);

    if(!$response->is_success()) {
      croak $response->status_line();
    }

    my $response_hash = from_json($response->content());
    $self->set_id_run($response_hash->{id_run});
    $self->set_is_paired($response_hash->{is_paired});

    my $read_one_lanes = $response_hash->{read_1};
    $self->set_read_one_lanes([]);

    foreach my $lane (@{$read_one_lanes}) {
      my $lane_obj = npg_qc::api::run::lane->new({util => $util});
      $lane_obj->id_run($self->id_run());
      $lane_obj->set_lane($lane->{lane});
      $lane_obj->set_read_end(1);
      push @{$self->get_read_one_lanes()}, $lane_obj;
      $lane_obj->create_tiles($lane->{tiles});
    }

    if ($response_hash->{is_paired}) {
      $self->set_id_run_pair($response_hash->{id_run_pair});
      $self->set_read_two_lanes([]);
      my $read_two_lanes = $response_hash->{read_2};
      foreach my $lane (@{$read_two_lanes}) {
        my $lane_obj = npg_qc::api::run::lane->new({util => $util});
        $lane_obj->id_run($self->get_id_run_pair());
        $lane_obj->set_lane($lane->{lane});
        $lane_obj->set_read_end(2);
        push @{$self->get_read_two_lanes()}, $lane_obj;
        $lane_obj->create_tiles($lane->{tiles});
      }
    }

    $self->read_summary();

    return 1;
  }

  sub read_summary {
    my ($self) = @_;
    my $summary = npg_qc::api::summary->new({util => $self->util()});
    $summary->id_run($self->id_run());
    $self->set_summary($summary);
    $summary->read_summary();
    return 1;
  }

  sub all_runs {
    my ($self) = @_;

    if (!$self->get_all_runs()) {
      my $util = $self->util();
      my $obj_uri    = sprintf '%s/%s', $util->base_uri(), 'main.json';
      my $response   = $util->get($obj_uri);

      if(!$response->is_success()) {
        croak $response->status_line();
      }

      my $response_hash = from_json($response->content());
      $self->set_all_runs($response_hash->{runs_with_data});
    }

    return $self->get_all_runs();
  }

}
1;
__END__
=head1 NAME

npg_qc::api::run

=head1 VERSION

$LastChangedRevision: 3510 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oRun = npg_qc::api::run->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_tile table

  my $aFields = $oRun->fields();

=head2 id_run - accessor for id_run. takes an id as an optional parameter (which it stores) and returns the id_run

  my $id_run = $oRun->id_run($id_run);

=head2 read_run_tiles - fetches all the run_tiles from the npg_qc application for this run and organises them into run::lane objects containing run::lane::tile objects. Also gets the run Summary for this run and stores it.

  $oRun->read_run_tiles();
  my $aLanesReadOne = $oRun->get_read_one_lanes();
  my $aLanesReadTwo = $oRun->get_read_two_lanes();

=head2 read_summary - fetches and stores the summary information for this run. Automatically run as part of read_run_tiles;

  $oRun->read_summary();
  $oSummary = $oRun->get_summary();

=head2 all_runs - fetches an array of all the runs with data in the NPG_QC database (note this is an unsorted list and includes id_run_pairs)

  my $aAllRuns = $oRun->all_runs();

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
