#########
# Author:        rmp
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-11-18 13:54:56 +0000 (Tue, 18 Nov 2008) $
# Id:            $Id: graph.pm 3510 2008-11-18 13:54:56Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-5.0/lib/npg_qc/graph.pm $
#
package npg_qc::graph;
use strict;
use warnings;
use GD::Graph::lines;
use GD::Graph::bars;
use GD::Graph::area;
use GD::Graph::mixed;
use GD;
use Sanger::Graphics::ColourMap;
use base qw(Class::Accessor);
use Readonly;
use Carp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3510 $ =~ /(\d+)/mx; $r; };

__PACKAGE__->mk_accessors(qw(cmap colours));

Readonly our $THREE          => 3;
Readonly our $FOUR           => 4;
Readonly our $WIDTH          => 400;
Readonly our $HEIGHT         => 200;
Readonly our $COLOR_ALLOCATE => 240;
Readonly our $LEGEND_FONT_SIZE => 10;

sub new {
  my ($class, $ref) = @_;
  $ref ||= {};
  bless $ref, $class;

  $ref->{cmap}    ||= Sanger::Graphics::ColourMap->new();
  $ref->{colours} ||= [qw(red purple orange blue green yellow magenta cyan)];
  return $ref;
}

sub array_rotate {
  my ($self, $in) = @_;

  my $h = scalar @{$in};
  $h or return [];
  my $w = scalar @{$in->[0]};
  $w or return [];

  my $out = [];

  for my $j (0..$h-1) {
    for my $i (0..$w-1) {
      $out->[$i]       ||= [];
      $out->[$i]->[$j]   = $in->[$j]->[$i];
    }
  }
  return $out;
}

sub plotter {
  my ($self, $data, $attrs, $type, $no_rotate) = @_;

  $type            ||= 'lines';
  $attrs           ||= {};
  $attrs->{legend} ||= [];

  my $width  = $attrs->{width}  || $WIDTH;
  my $height = $attrs->{height} || $HEIGHT;

  if(!scalar @{$data}) {
    my $title = "No @{[$attrs->{title}||q()]} Data";
    my $gd    = GD::Image->new($width, $height);
    $gd->colorAllocate($COLOR_ALLOCATE,$COLOR_ALLOCATE,$COLOR_ALLOCATE);
    my $blk   = $gd->colorAllocate(0,0,0);
    $gd->string(gdSmallFont,
		$width/2 - (length $title) * $THREE,
		$height/2 - $FOUR,
		$title,
		$blk);
    return $gd->png();
  }

  my $cmap = $self->cmap();
  my $gtype = "GD::Graph::$type";
  my $graph = $gtype->new($width, $height);

  $graph->set(
              dclrs        => [
			       map {
				 GD::Graph::colour::add_colour(q(#).$cmap->hex_by_name($_));
			       } @{$self->colours()}
			      ],
              fgclr        => 'lgray',
              boxclr       => 'white',
              accentclr    => 'black',
              shadowclr    => 'black',
              y_long_ticks => 1,
              %{$attrs},
             );

  if(scalar @{$attrs->{legend}}) {
    $graph->set_legend(@{$attrs->{legend}});
    $graph->set_legend_font(
        ['verdana', 'arial', gdMediumBoldFont], $LEGEND_FONT_SIZE )
  }

  if (!$no_rotate) {
    $data = $self->array_rotate($data);
  }
  my $png;
  eval { $png = $graph->plot($data)->png(); } or do { croak $graph->error(); };
  return $png;
}

1;

__END__

=head1 NAME

npg_qc::graph

=head1 VERSION

$LastChangedRevision: 3510 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new

=head2 array_rotate

=head2 plotter

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

GD::Graph::lines
GD::Graph::bars
GD::Graph::area
GD::Graph::mixed
GD
Sanger::Graphics::ColourMap
Class::Accessor

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Roger Pettett, E<lt>rmp@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Roger Pettett

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
