#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-30
# Last Modified: $Date: 2008-11-18 13:54:56 +0000 (Tue, 18 Nov 2008) $
# Id:            $Id: swift_summary.pm 3510 2008-11-18 13:54:56Z gq1 $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-5.0/lib/npg_qc/model/swift_summary.pm $
#

package npg_qc::model::swift_summary;
use strict;
use warnings;
use base qw(npg_qc::model);
use Statistics::Lite qw(:all);
use Carp;
use English qw(-no_match_vars);
use npg_qc::model::swift_report;
use npg_qc::model::swift_all_intensities_per_cycle_per_base;
use npg_qc::model::swift_called_intensities_per_cycle_per_base;
use npg_qc::model::swift_pf_errors_per_cycle;
use npg_qc::model::swift_npf_errors_per_cycle;
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3510 $ =~ /(\d+)/mx; $r; };

Readonly our $LAST_OF_ARRAY => -1;

sub cycle_count {
  my ($self) = @_;
  if (!$self->{cycle_count}) {
    my $ref;
    my $query = q{SELECT MAX(cycle) FROM swift_pf_errors_per_cycle spe, run_tile rt WHERE rt.id_run = ? AND rt.id_run_tile = spe.id_run_tile};
    eval {
      $ref = $self->util->dbh->selectall_arrayref($query, {}, $self->{id_run});
      1;
    } or do {
      croak $EVAL_ERROR;
    };
    $self->{cycle_count} = $ref->[0]->[0];
  }
  return $self->{cycle_count};
}

sub tiles {
  my ($self) = @_;
  if(!$self->{tiles}) {
    my $dbh = $self->util->dbh();
    my $query = q{SELECT DISTINCT position, tile FROM run_tile WHERE id_run = ? ORDER BY position, tile};
    eval {
      my $sth = $dbh->prepare($query);
      $sth->execute($self->{id_run});
      while (my $href = $sth->fetchrow_hashref()) {
        push @{$self->{tiles}}, $href;
      }
      1;
    } or do {
      croak $EVAL_ERROR;
    };
  }
  return $self->{tiles};
}

sub aggregated_swift_report {
  my ($self) = @_;
  if (!$self->{aggregated_swift_report}) {
    my $sr = npg_qc::model::swift_report->new({util => $self->util()});
    $self->{aggregated_swift_report} = $sr->aggregated_swift_reports($self->{id_run});
  }
  return $self->{aggregated_swift_report};
}

sub average_all_intensities {
  my ($self) = @_;
  if (!$self->{average_all_intensities}) {
    my $sai = npg_qc::model::swift_all_intensities_per_cycle_per_base->new({util => $self->util()});
    $self->{average_all_intensities}  = $sai->average_intensities_over_cycle($self->{id_run});
  }
  return $self->{average_all_intensities};
}

sub average_called_intensities {
  my ($self) = @_;
  if (!$self->{average_called_intensities}) {
    my $sci = npg_qc::model::swift_called_intensities_per_cycle_per_base->new({util => $self->util()});
    $self->{average_called_intensities} = $sci->average_intensities_over_cycle($self->{id_run});
  }
  return $self->{average_called_intensities};
}

sub pf_total_errors_over_cycle {
  my ($self) = @_;
  if (!$self->{pf_total_errors_over_cycle}) {
    my $pfe = npg_qc::model::swift_pf_errors_per_cycle->new({util => $self->util()});
    $self->{pf_total_errors_over_cycle} = $pfe->total_errors_over_cycle($self->{id_run}, $self->cycle_count(), $self->aggregated_swift_report());
  }
  return $self->{pf_total_errors_over_cycle};
}

sub npf_total_errors_over_cycle {
  my ($self) = @_;
  if (!$self->{npf_total_errors_over_cycle}) {
    my $npfe = npg_qc::model::swift_npf_errors_per_cycle->new({util => $self->util()});
    $self->{npf_total_errors_over_cycle} = $npfe->total_errors_over_cycle($self->{id_run}, $self->cycle_count(), $self->aggregated_swift_report());
  }
  return $self->{npf_total_errors_over_cycle};
}

sub all {
  my ($self, $position) = @_;
  my $ave_all_int = $self->average_all_intensities();
  return $self->all_call_refactor($position, $ave_all_int);
}

sub call {
  my ($self, $position) = @_;
  my $ave_called_int = $self->average_called_intensities();
  return $self->all_call_refactor($position, $ave_called_int);
}

sub pf_errors {
  my ($self, $position) = @_;
  my $pf_errors = $self->pf_total_errors_over_cycle();
  return $self->errors_refactor($position, $pf_errors);
}


sub npf_errors {
  my ($self, $position) = @_;
  my $npf_errors = $self->npf_total_errors_over_cycle();
  return $self->errors_refactor($position, $npf_errors);
}

sub all_call_refactor {
  my ($self, $position, $arrayref) = @_;
  my $wanted = [];
  foreach my $cycle (@{$arrayref}) {
    next if ($cycle->[0] != $position);
    my @temp = @{$cycle};
    shift @temp;
    push @{$wanted}, \@temp;
  }
  return $wanted;
}

sub errors_refactor {
  my ($self, $position, $arrayref) = @_;
  my $wanted = $self->all_call_refactor($position, $arrayref);
  foreach my $cycle (@{$wanted}) {
    $cycle = [$cycle->[0], $cycle->[$LAST_OF_ARRAY]];
  }
  return $wanted;
}

sub tile_pf_errors {
  my ($self, $position, $tile) = @_;
  my $arg_refs = {
    method   => 'swift_pf_errors_per_cycles',
    position => $position,
    tile     => $tile,
  };
  return $self->tile_errors_refactor($arg_refs);
}

sub tile_npf_errors {
  my ($self, $position, $tile) = @_;
  my $arg_refs = {
    method   => 'swift_npf_errors_per_cycles',
    position => $position,
    tile     => $tile,
  };
  return $self->tile_errors_refactor($arg_refs);
}

sub tile_errors_refactor {
  my ($self, $arg_refs) = @_;
  my $method = $arg_refs->{method};
  my $run_tile = npg_qc::model::run_tile->new({
    util     => $self->util(),
    id_run   => $self->{id_run},
    position => $arg_refs->{position},
    tile     => $arg_refs->{tile},
    end      => 1,
  });
  my $cycle_errors = $run_tile->$method();
  foreach my $cycle (@{$cycle_errors}) {
    my $cycle_no = sprintf '%02d', $cycle->cycle();
    $cycle = [$cycle_no, $cycle->errors_as_percentage($self->cycle_count())];
  }
  my @return_array = sort { $a->[0] <=> $b->[0] } @{$cycle_errors};
  return \@return_array;
}

1;
__END__
=head1 NAME

npg_qc::model::swift_summary

=head1 VERSION

$LastChangedRevision: 3510 $

=head1 SYNOPSIS

  my $oSwiftSummary = npg_qc::model::swift_summary->new({util => $util});

=head1 DESCRIPTION

Base model to sit under npg_qc::view::summary

=head1 SUBROUTINES/METHODS

=head2 cycle_count - returns the cycle count for given id_run

  my $sCycleCount = $oSwiftSummary->cycle_count();

=head2 lanes - returns an array of the lanes numbers stored for given id_run

  my $aLanes = $oSwiftSummary->lanes();

=head2 tiles - returns an array of the tiles for given id_run

  my $aTiles = $oSwiftSummary->tiles();

=head2 aggregated_swift_report - sums up and returns an array of report stats for each lane on given run

  my $aAggregatedSwiftReport = $oSwiftSummary->aggregated_swift_report();

=head2 average_all_intensities - averages all the tile intensities for all bases for a cycle (per lane)

  my $aAverageAllIntensities = $oSwiftSummary->average_all_intensities();
  
=head2 average_called_intensities - averages all the tile intensities for called bases for a cycle (per lane)

  my $aAverageCalledIntensities = $oSwiftSummary->average_called_intensities();
  
=head2 pf_total_errors_over_cycle - totals all the pf errors over the tiles for a cycle (per lane)

  my $aPFTotalErrorsOverCycle = $oSwiftSummary->pf_total_errors_over_cycle();

=head2 npf_total_errors_over_cycle - totals all the pf errors over the tiles for a cycle (per lane)

  my $aNPFTotalErrorsOverCycle = $oSwiftSummary->npf_total_errors_over_cycle();

=head2 all - sorts out data for graph plotting
=head2 call - sorts out data for graph plotting
=head2 all_call_refactor - refactor of common code for all and call methods

  my $aAll  = $oSwiftSummary->all($position);
  my $aCall = $oSwiftSummary->call($position); 

=head2 pf_errors - sorts out data for graph plotting
=head2 npf_errors - sorts out data for graph plotting
=head2 errors_refactor - refactor of common code for pf_errors and npf_errors methods

  my $aPFErrors  = $oSwiftSummary->pf_errors($position);
  my $aNPFErrors = $oSwiftSummary->npf_errors($position);

=head2 tile_pf_errors - sorts out data for graph plotting
=head2 tile_npf_errors - sorts out data for graph plotting
=head2 tile_errors_refactor - refactor of common code for tile_pf_errors and tile_npf_errors methods

  my $aTilePFErrors  = $oSwiftSummary->tile_pf_errors($position);
  my $aTileNPFErrors = $oSwiftSummary->tile_npf_errors($position);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
English
Carp
Statistics::Lite

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
