#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2008-09-23
# Last Modified: $Date: 2008-11-18 13:54:56 +0000 (Tue, 18 Nov 2008) $
# Id:            $Id: 10-model-cache_query.t 3510 2008-11-18 13:54:56Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-5.0/t/10-model-cache_query.t $
#

use strict;
use warnings;
use Test::More tests => 38;
use English qw(-no_match_vars);
use SangerPaths qw(core tracking badger ensembl);
use IO::Scalar;
use lib qw{t};
use t::util;
use npg_qc::model::cache_query;
use Data::Dumper;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3510 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::model::cache_query');


my $util = t::util->new({fixtures =>1});


{
  my $model = npg_qc::model::cache_query->new({util => $util});
  isa_ok($model, 'npg_qc::model::cache_query', '$model');
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    ssha_sql => 'GbdhOrKXza98/jXuv9WAzNW0pcQ=',
  });
  eval { $model->save() };
  like($EVAL_ERROR, qr{Column\ \'id_run\'\ cannot\ be\ null}, 'field id_run in table cache_query cannot be null');  
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    ssha_sql => 'GbdhOrKXza98/jXuv9WAzNW0pcQ=',
    id_run => 917,
  });
  eval { $model->save() };
  like($EVAL_ERROR, qr{Column\ \'end\'\ cannot\ be\ null}, 'field end in table cache_query cannot be null');  
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    ssha_sql => 'GbdhOrKXza98/jXuv9WAzNW0pcQ=',
    id_run => 917,
    end    => 1,
  });
  eval { $model->save() };
  like($EVAL_ERROR, qr{Column\ \'type\'\ cannot\ be\ null}, 'field type in table cache_query cannot be null');  
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    ssha_sql => 'GbdhOrKXza98/jXuv9WAzNW0pcQ=',
    id_run => 917,
    end    => 1,
    type   => 'num_tiles_z_alert',
  });
  eval { $model->save() };
  like($EVAL_ERROR, qr{Column\ \'results\'\ cannot\ be\ null}, 'field results in table cache_query cannot be null');  
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    ssha_sql => 'GbdhOrKXza98/jXuv9WAzNW0pcQ=',
    id_run => 917,
    end    => 1,
    type   => 'num_tiles_z_alert',
    results => 'aaa',
  });
  eval { $model->save() };
  like($EVAL_ERROR, qr{Column\ \'is_current\'\ cannot\ be\ null}, 'field is_current in table cache_query cannot be null');  
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    ssha_sql => 'GbdhOrKXza98/jXuv9WAzNW0pcQ=',
    id_run => 917,
    end    => 1,
    type   => 'num_tiles_z_alert',
    results => 'aaa',
    is_current => 1,
  });
  eval { $model->save() };
  is($EVAL_ERROR, q{}, 'save ok');
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    });
  my $sql = "SELECT rt.position AS lane,
                         lqc.end AS end,
                         ROUND(SUM(lqc.clusters_raw*lqc.perc_pf_clusters*?/?)) AS lane_yield,
                         ROUND(AVG(lqc.clusters_raw*lqc.perc_pf_clusters/?)) AS clusters_pf,
                         ROUND(STD(lqc.clusters_raw*lqc.perc_pf_clusters/?)) AS clusters_pf_sd,
                         ROUND(AVG(lqc.clusters_raw)) AS clusters_raw,
                         ROUND(STD(lqc.clusters_raw)) AS clusters_raw_sd,
                         ROUND(AVG(lqc.av_1st_cycle_int_pf)) AS first_cycle_int,
                         ROUND(STD(lqc.av_1st_cycle_int_pf)) AS first_cycle_int_sd,
                         ROUND(AVG(lqc.av_perc_intensity_after_20_cycles_pf),2) AS perc_int_20_cycles,
                         ROUND(STD(lqc.av_perc_intensity_after_20_cycles_pf),2) AS perc_int_20_cycles_sd,
                         ROUND(AVG(lqc.perc_pf_clusters),2) AS perc_pf_clusters,
                         ROUND(STD(lqc.perc_pf_clusters),2) AS perc_pf_clusters_sd,
                         ROUND(AVG(lqc.perc_align_pf),2) AS perc_pf_align,
                         ROUND(STD(lqc.perc_align_pf),2) AS perc_pf_align_sd,
                         ROUND(AVG(lqc.av_alignment_score_pf),2) AS align_score,
                         ROUND(STD(lqc.av_alignment_score_pf),2) AS align_score_sd,
                         ROUND(AVG(lqc.perc_error_rate_pf),2) AS perc_error_rate,
                         ROUND(STD(lqc.perc_error_rate_pf),2) AS perc_error_rate_sd
                   FROM  lane_qc lqc, run_tile rt
                   WHERE rt.id_run = ?
                   AND   rt.id_run_tile = lqc.id_run_tile
                   AND   lqc.end = ?
                   GROUP BY rt.position, lqc.end
                   ORDER BY position";
  $model->generate_ssha($sql);
  is($model->generate_ssha($sql), 'GbdhOrKXza98/jXuv9WAzNW0pcQ=' , 'get correct ssha key');
}

{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'movez_tiles',
  });
  eval { $model->cache_movez_tiles() };
  is($EVAL_ERROR, q{}, 'cache movez tiles');
  
  eval{ $model->get_cache_by_id_type_end()};
  is($EVAL_ERROR, q{}, 'get cache movez tiles');
  
  is($model->get_cache_by_id_type_end()->[0][0], 3, 'get correct cache movez tiles');
}
###
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'cycle_count',
  });
  eval { $model->cache_cycle_count() };
  is($EVAL_ERROR, q{}, 'cache cycle count');
  
  eval{ $model->get_cache_by_id_type_end()};
  is($EVAL_ERROR, q{}, 'get cache cycle count');
  
  is($model->get_cache_by_id_type_end()->[0][0], 37, 'get correct cache cycle count');
}
###

{
  my $model = npg_qc::model::cache_query->new({
     util => $util,
     id_run =>3,
     type   => 'cycle_count',
     end    => 1,
     });
  my $id_runs = $model->check_cycle_count();
  
}


###
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'lane_summary',
  });
  eval { $model->cache_lane_summary(37) };
  is($EVAL_ERROR, q{}, 'cache lane summary');
  
  my $dump;
  eval{
   my $rows_ref = $model->get_cache_by_id_type_end();
   $dump = new Data::Dumper([$rows_ref],['rows_ref']);
   $dump->Indent(0);
   1; 
  };
  is($EVAL_ERROR, q{}, 'get lane summary');
  
  is($dump->Dump, q{$rows_ref = [{'clusters_pf' => '66598','lane_yield' => '7392','perc_pf_clusters_sd' => '0.00','perc_pf_align_sd' => '0.00','clusters_pf_sd' => '0','perc_pf_clusters' => '79.88','align_score' => '0.00','perc_pf_align' => '0.00','perc_error_rate' => '0.00','first_cycle_int_sd' => '0','perc_error_rate_sd' => '0.00','clusters_raw_sd' => '0','perc_int_20_cycles_sd' => '0.00','first_cycle_int' => '159','lane' => '1','clusters_raw' => '83372','end' => '1','perc_int_20_cycles' => '99.06','align_score_sd' => '0.00'},{'clusters_pf' => '66598','lane_yield' => '2464','perc_pf_clusters_sd' => '0.00','perc_pf_align_sd' => '0.00','clusters_pf_sd' => '0','perc_pf_clusters' => '79.88','align_score' => '0.00','perc_pf_align' => '0.00','perc_error_rate' => '0.00','first_cycle_int_sd' => '0','perc_error_rate_sd' => '0.00','clusters_raw_sd' => '0','perc_int_20_cycles_sd' => '0.00','first_cycle_int' => '159','lane' => '2','clusters_raw' => '83372','end' => '1','perc_int_20_cycles' => '99.06','align_score_sd' => '0.00'}];}, 'get correct cache summary');
}

###
{
  my $model = npg_qc::model::cache_query->new({util => $util});
  my $id_runs = $model->get_runs_cache_movez_tiles();
  is (scalar @{$id_runs}, 3, 'get id_runs for caching movez_tiles');
  
  eval{
    $model->cache_movez_tiles_all();
    1;
  };
  is($EVAL_ERROR, q{}, 'cache movez tiles all');
}

###
{
  my $model = npg_qc::model::cache_query->new({util => $util});
  my $id_runs = $model->get_runs_cache_lane_summary();
  ok($id_runs, 'get id_runs and ends for caching lane summary');
  
  eval{
    $model->cache_lane_summary_all();
    1;
  };
  is($EVAL_ERROR, q{}, 'cache lane summary all');
  $id_runs = $model->get_runs_cache_lane_summary();
  is($id_runs, undef, 'get id_runs and ends for caching lane summary - none returned');

  $model->id_run(10);
  $model->end(1);
  $model->type('lane_summary');

  my $rows_ref;
  eval{
   $rows_ref = $model->get_cache_by_id_type_end();
   1; 
  };
  is($EVAL_ERROR, q{}, 'get lane summary');
  
  is(scalar@{$rows_ref}, 8, 'correct number of elements in returned array');
  isa_ok($rows_ref->[0], 'HASH', 'first element');

  my $keys_string = join ':', sort keys %{$rows_ref->[0]};
  is($keys_string, 'align_score:align_score_sd:clusters_pf:clusters_pf_sd:clusters_raw:clusters_raw_sd:end:first_cycle_int:first_cycle_int_sd:lane:lane_yield:perc_error_rate:perc_error_rate_sd:perc_int_20_cycles:perc_int_20_cycles_sd:perc_pf_align:perc_pf_align_sd:perc_pf_clusters:perc_pf_clusters_sd', 'keys are correct in first element hash');
}

###
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'lane_summary',
  });
  
  my $cached;
  eval{
    $cached = $model->is_current_cached();
    1;
  };
  
  is($EVAL_ERROR, q{}, 'check cached or not');
  
  is($cached, 1, 'query already cached');
  
  $model->id_run(111);
  
  is($model->is_current_cached(), 0, 'query not current cached');
}
##
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'lane_summary',
  });
  
  my $updated;
  eval{
    $updated = $model->set_to_not_current();
    1;
  };
  
  is($EVAL_ERROR, q{}, 'set_to_not_current');
  
  is($updated, 1, '1 row updated');
  
  is($model->is_current_cached(), 0, 'query not current cached');

}


##
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'movez_tiles',
  });
  
  is($model->id_cache_query(), 3, 'get current cache primary key');

}

##
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'movez_tiles',
  });

  eval{
    $model->update_current_cache();
    1;
  };
  is($EVAL_ERROR, q{}, 'update current cache movez_tiles');
}

##
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'lane_summary',
  });

  eval{
    $model->update_current_cache();
    1;
  };
  is($EVAL_ERROR, q{}, 'update current cache lane_summary');
}

##
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'movez_tiles',
  });

  eval{
    $model->cache_new_copy_data();
    1;
  };
  is($EVAL_ERROR, q{}, 'save new copy of cache movez_tiles');
}

##
{
  my $model = npg_qc::model::cache_query->new({
    util => $util,
    id_run => 3,
    end    => 1,
    type   => 'lane_summary',
  });

  eval{
    $model->cache_new_copy_data();
    1;
  };
  is($EVAL_ERROR, q{}, 'save new copy of cache lane_summary');
}
